;;; Definim la classe Animals, amb els atributs de tots els animals.
;;; Aquesta classe penja de la classe USER.
;;; Amb (create-accessor read-write) permetem el donar valors inicials 
;;; a altura, pes i color quan s'inicialitza una instancia.

(defclass Animals
   (is-a USER)                           
   (role abstract)                      
   (slot altura (type FLOAT) (range 0.0 15.0) 
               (create-accessor read-write))
   (slot pes (type INTEGER) (range 0 10000) 
               (create-accessor read-write))
   (slot color (type SYMBOL) (allowed-values Groc Blau Blanc Negre Verd) 
               (create-accessor read-write))
   (slot vola? (type SYMBOL) (allowed-values Si No))
   (slot neda? (type SYMBOL) (allowed-values Si No))
)

;;; Definicio de la classe AnimalsAquatics.
;;; Posem valors per defecte als slots vola? i neda?. Aquests valors
;;; es guardaran a nivell de classe, i no es podran modificar.
;;; Afegim un altre slot pel nombre d'aletes; es pot donar valor a
;;; aquest slot quan s'inicialitza una instancia.

(defclass AnimalsAquatics
   (is-a Animals)
   (role abstract)
   (slot vola? (storage shared) (access read-only) (default No))
   (slot neda? (storage shared) (access read-only) (default Si))
   (slot naletes (type INTEGER) (range 0 10) (create-accessor read-write))
)

;;; Definicio de la classe AnimalsTerrestres.
;;; Tambe posem valors per defecte als slots vola? i neda?. En aquest
;;; cas, es guardaran a nivell d'instancia i seran modificables.

(defclass AnimalsTerrestres
   (is-a Animals)
   (role abstract)
   (slot vola? (default No))
   (slot neda? (default No))
)

;;; Definicio de la classe AnimalsVoladors.
;;; Tambe tenim valors per defecte als slots vola? i neda?. El primer
;;; es guardaria a nivell de classe i no es podria modificar, mentre
;;; que el segon es guardaria a nivell d'instancia i si es podria modificar.

(defclass AnimalsVoladors
   (is-a Animals)
   (role abstract)
   (slot vola? (storage shared) (access read-only) (default Si))
   (slot neda? (default No))
)

;;; Definim la classe Dofins com a subclasse d'AnimalsAquatics.
;;; Es poden fer instancies d'aquesta classe.

(defclass Dofins
   (is-a AnimalsAquatics)
   (role concrete)
)

;;; Definim la classe Balenes com a subclasse d'AnimalsAquatics.
;;; Donem valors per defecte als slots altura, pes i naletes. Aquest
;;; ultim es guarda a nivell de classe, i no es pot modificar.
;;; Amb (source composite) s'aconsegueix que es tinguin en compte
;;; els facets d'altura i pes definits a les superclasses.

(defclass  Balenes
   (is-a AnimalsAquatics)
   (role abstract)
   (slot altura (default 5.5) (source composite))
   (slot pes (default 3000) (source composite))
   (slot naletes (storage shared) (access read-only) (default 4))
)

;;; Definim les BalenesBlaves com a subclasse de Balenes. Aquest tipus
;;; de balenes sempre tindran color blau.

(defclass BalenesBlaves
   (is-a Balenes)
   (role concrete)
   (slot color (default Blau) (storage shared) (access read-only))
)

;;; Les BalenesArtiques tambe son una subclasse de Balenes. Tenen uns
;;; valors per defecte al pes i l'altura superiors a les balenes normals.

(defclass BalenesArtiques
   (is-a Balenes)
   (role concrete)
   (slot pes (default 5000) (source composite))
   (slot altura (default 7.0) (source composite))
)
 