;;; Classe d'objectes: PersonatgeFantastic
;;; No es poden fer instncies d'aquesta classe
;;; Atributs: fora, intel.ligencia, potencia i arma
;;; Potencia = fora * intel.ligencia

;;; Cal posar (visibility public) a aquells slots que
;;; hagin de ser llegits des de manegadors de missatges
;;; enviats a objectes pertanyents a alguna subclasse.

(defclass PersonatgeFantastic
  (is-a USER)
  (role abstract)
  (slot forza (type INTEGER) (range 0 10) (create-accessor write) (visibility public))
  (slot inteligencia (type INTEGER) (range 0 5) (create-accessor write) (visibility public))
  (slot potencia (type INTEGER) (range 0 50) (visibility public))
  (slot arma (type SYMBOL) (visibility public))
)

;;; Subclasse: Hobbits. Habitualment tenen fora 6 i intel.ligncia 3.
;;; Es poden fer instncies d'aquesta classe.

(defclass Hobbits
  (is-a PersonatgeFantastic)
  (role concrete)
  (slot forza (default 6) (source composite))
  (slot inteligencia (default 3) (source composite))
)

;;; Subclasse: Elfs. Habitualment tenen fora 7 i intel.ligncia 5.
;;; Tots utilitzen arcs.
;;; Es poden fer instncies d'aquesta classe.

(defclass Elfs
   (is-a PersonatgeFantastic)
   (role concrete)
   (slot forza (default 7) (source composite))
   (slot inteligencia (default 4) (source composite))
   (slot arma (default Arc) (access read-only) (storage shared) (source composite))
)

;;; Subclasse: Nans.
;;; Tots utilitzen destrals.
;;; Espoden fer instancies d'aquesta classe.

(defclass Nans
   (is-a PersonatgeFantastic)
   (role concrete)
   (slot arma (default Destral) (access read-only) (storage shared) (source composite))
)

;;; Quan es llegeix un slot d'un objecte que no s instncia
;;; de la classe on est definit el manegador, sin
;;; d'una subclasse, cal fer-ho amb (dynamic-get nom-slot)

(defmessage-handler PersonatgeFantastic llegir-atribut-forza ()
   (dynamic-get forza)
)

(defmessage-handler PersonatgeFantastic llegir-atribut-inteligencia ()
   (dynamic-get inteligencia)
)

(defmessage-handler PersonatgeFantastic init after ()
   (bind ?self:potencia 
       (* (send ?self llegir-atribut-forza)
          (send ?self llegir-atribut-inteligencia)
       )
   )
)

(defmessage-handler PersonatgeFantastic posa-arma (?a)
  (bind ?self:arma ?a)
)

(defmessage-handler Elfs print ()
   (printout T (instance-name ?self) " te com a arma: " ?self:arma crlf)
   (printout T "La seva fora es " ?self:forza)
   (printout T " i la seva intel.ligencia es " ?self:inteligencia "." crlf)
   (printout T "Per tant, la seva potencia es de " ?self:potencia "." crlf)
)

